using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace Calendar
{
    public abstract class AbstractRenderer
    {
        protected readonly object m_SyncObj = new object();

        protected KS.Gantt.Gantt m_GanttControl = null;
        public KS.Gantt.Gantt GanttControl
        {
            get
            {
                return m_GanttControl;
            }
            set
            {
                m_GanttControl = value;
            }
        }
        
        public virtual void ResetFonts()
        {
            lock (m_SyncObj)
            {
                if (hourFont != null)
                {
                    hourFont.Dispose();
                    hourFont = null;
                }

                if (hourFontBold != null)
                {
                    hourFontBold.Dispose();
                    hourFontBold = null;
                }

                if (minuteFont != null)
                {
                    minuteFont.Dispose();
                    minuteFont = null;
                }
            }
        }

        public virtual Color AllDayEventsBackColor
        {
            get
            {
                return InterpolateColors(this.BackColor, Color.Black, 0.5f);
            }
        }

        public event EventHandler BaseFontChanged;
        public void OnBaseFontChanged()
        {
            if (BaseFontChanged != null)
                BaseFontChanged(this, EventArgs.Empty);
        }


        //protected Font baseFont;
        public virtual Font BaseFont
        {
            get
            {
                return SystemFonts.IconTitleFont;                
            }            
        }

        public virtual Color HourSeperatorColor
        {
            get
            {
                return System.Drawing.Color.FromArgb(234, 208, 152);
            }
        }

        public virtual Color HalfHourSeperatorColor
        {
            get
            {
                return System.Drawing.Color.FromArgb(243, 228, 177);
            }
        }

        public virtual Color HourColor
        {
            get
            {
                // ToDo: Set Weekend Color from Gantt-Control
                if (m_GanttControl != null)
                    return m_GanttControl.WeekendFormatStyle.BackgroundStyle.Color;
                else
                    return System.Drawing.Color.FromArgb(255, 244, 188);
            }
        }

        public virtual Color WorkingHourColor
        {
            get
            {
                // ToDo: Set WorkingHour / Background Color from Gantt Control
                if (m_GanttControl != null)
                    return m_GanttControl.BackColor;
                else
                    return System.Drawing.Color.FromArgb(255, 255, 213);
            }
        }

        public virtual Color BackColor
        {
            get
            {
                return SystemColors.Control;
            }
        }

        public virtual Color HoursBackColor
        {
            get
            {
                if (m_GanttControl != null)
                    return m_GanttControl.WeekendFormatStyle.BackgroundStyle.Color;
                else
                    return BackColor;
            }
        }

        public virtual Color SelectionColor
        {
            get
            {
                return SystemColors.Highlight;                
            }
        }

        protected Font hourFont;
        public virtual Font HourFont
        {
            get
            {
                if (hourFont == null)
                {
                    hourFont = new Font(SystemFonts.IconTitleFont.FontFamily, 16f, FontStyle.Regular);
                }

                return hourFont;
            }
            set
            {
                hourFont = value;
            }
        }

        protected Font hourFontBold;
        public virtual Font HourFontBold
        {
            get
            {
                if (hourFontBold == null)
                {
                    hourFontBold = new Font(SystemFonts.IconTitleFont.FontFamily, 16f, FontStyle.Bold);
                }

                return hourFontBold;
            }
        }

        private Font minuteFont;
        public virtual Font MinuteFont
        {
            get
            {
                if (minuteFont == null)
                {
                    minuteFont = new Font(SystemFonts.IconTitleFont.FontFamily, 9f, FontStyle.Regular);
                }

                return minuteFont;
            }
            set
            {
                minuteFont = value;
            }
        }

        public abstract void DrawHourLabel(Graphics g, Rectangle rect, int hour);

        public abstract void DrawDayHeader(Graphics g, Rectangle rect, DateTime date);

        public abstract void DrawUserHeader(Graphics g, Rectangle rect, string UserName);

        public abstract void DrawDayBackground(Graphics g, Rectangle rect);

        public virtual void DrawHourRange(Graphics g, Rectangle rect, bool drawBorder, bool hilight)
        {
            using (SolidBrush brush = new SolidBrush(hilight ? this.SelectionColor : this.WorkingHourColor))
            {
                g.FillRectangle(brush, rect);
            }

            if (drawBorder)
                g.DrawRectangle(SystemPens.WindowFrame, rect);
        }

        public virtual void DrawDayGripper(Graphics g, Rectangle rect, int gripWidth)
        {
            using (Brush m_Brush = new SolidBrush(Color.White))
                g.FillRectangle(m_Brush, rect.Left, rect.Top - 1, gripWidth, rect.Height);

            using (Pen m_Pen = new Pen(Color.Black))
                g.DrawRectangle(m_Pen, rect.Left, rect.Top - 1, gripWidth, rect.Height);
        }

        public abstract void DrawAppointment(Graphics g, Rectangle rect, Appointment appointment, bool isSelected, int gripWidth, ImageList imageList);

        public void DrawAllDayBackground(Graphics g, Rectangle rect)
        {
            using (Brush brush = new SolidBrush(InterpolateColors(this.BackColor, Color.Black, 0.5f)))
                g.FillRectangle(brush, rect);
        }

        public static Color InterpolateColors(Color color1, Color color2, float percentage)
        {
            int num1 = ((int)color1.R);
            int num2 = ((int)color1.G);
            int num3 = ((int)color1.B);
            int num4 = ((int)color2.R);
            int num5 = ((int)color2.G);
            int num6 = ((int)color2.B);
            byte num7 = Convert.ToByte(((float)(((float)num1) + (((float)(num4 - num1)) * percentage))));
            byte num8 = Convert.ToByte(((float)(((float)num2) + (((float)(num5 - num2)) * percentage))));
            byte num9 = Convert.ToByte(((float)(((float)num3) + (((float)(num6 - num3)) * percentage))));
            return Color.FromArgb(num7, num8, num9);
        }
    }
}
