using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Data.SqlClient;
using System.Data.OleDb;
using System.Text;

namespace KS.Common
{
    public enum SQLBuilderTypes
    {
        sqlString = 0,
        sqlBool = 1,
        sqlInt = 2,
        sqlFloat = 3,
        sqlDate = 4,
        sqlMemo = 5,
        sqlChar = 6,
        sqlMoney = 7
    }

    public enum DBMSTypes
    {
        MSSQLServer = 0,
        MSAccess = 1
    }

    public enum SQLServerBoolTypes
    {
        boolBit = 0,
        boolTinyInt = 1
    }

    public class SQLColumnDefinition
    {
        public SQLBuilderTypes SQLType;
        public bool AllowNull;
        public int MaxLen;
        public object Value;
        public object DefaultValue;
        public bool bActive = true;
        public bool bWhere = false;

        public SQLColumnDefinition()
        {
            Reset();
        }

        public void Reset()
        {
            SQLType = SQLBuilderTypes.sqlString;
            AllowNull = true;
            MaxLen = 255;
            Value = null;
            bActive = true;
            bWhere = false;
        }

        public string DBValueString(DBMSTypes DBMSType, System.Globalization.NumberFormatInfo nfi)
        {
            string strValue;

            if ((Value == null || Value.Equals(DBNull.Value)) && this.AllowNull)
                return "NULL";

            switch (SQLType)
            {
                case SQLBuilderTypes.sqlString:
                case SQLBuilderTypes.sqlChar:
                    strValue = Value.SafeString();
                    if (MaxLen > 0)
                        strValue = Strings.StrLeft(strValue, MaxLen);

                    if (AllowNull)
                        strValue = Strings.ConvertNullSQL(strValue);
                    else
                        strValue = "\'" + Strings.DoubleQuotes(strValue) + "\'";

                    break;

                case SQLBuilderTypes.sqlInt:
                    if (Value.IsNumeric())
                        strValue = Value.SafeInt().ToString(nfi);
                    else
                    {
                        if (AllowNull)
                            strValue = "NULL";
                        else
                            strValue = DefaultValue.SafeInt().ToString(nfi);
                    }

                    break;

                case SQLBuilderTypes.sqlFloat:                
                    if (Value.IsNumeric())
                    {
                        strValue = Value.SafeDouble().ToString(nfi);
                    }
                    else
                    {
                        if (AllowNull)
                            strValue = "NULL";
                        else
                            strValue = DefaultValue.SafeDouble().ToString(nfi);
                    }

                    break;

                case SQLBuilderTypes.sqlMoney:
                    if (Value.IsNumeric())
                    {
                        strValue = Value.SafeDecimal().ToString(nfi);
                    }
                    else
                    {
                        if (AllowNull)
                            strValue = "NULL";
                        else
                            strValue = DefaultValue.SafeDecimal().ToString(nfi);
                    }

                    break;

                case SQLBuilderTypes.sqlBool:
                    if (DBMSType == DBMSTypes.MSSQLServer)
                        strValue = Value.SafeBool().ToInt().ToString();
                    else
                        strValue = Value.SafeBool().ToString(nfi).ToUpper();
                    break;


                case SQLBuilderTypes.sqlDate:
                    if (Value.IsDate())
                    {
                        if (AllowNull && (DateTime)Value == DateTime.MinValue)
                            strValue = "NULL";
                        else
                            strValue = Strings.SQLDateTime(System.Convert.ToDateTime(Value), DBMSType == DBMSTypes.MSSQLServer);
                    }
                    else
                    {
                        if (AllowNull || !DefaultValue.IsDate())
                            strValue = "NULL";
                        else
                            strValue = Strings.SQLDateTime(System.Convert.ToDateTime(DefaultValue), DBMSType == DBMSTypes.MSSQLServer);
                    }
                    break;

                case SQLBuilderTypes.sqlMemo:
                    strValue = Value.SafeString();
                    if (AllowNull)
                        strValue = Strings.ConvertNullSQL(strValue);
                    else
                        strValue = "\'" + Strings.DoubleQuotes(strValue) + "\'";

                    break;

                default:
                    strValue = "NULL";
                    break;
            }

            return strValue;
        }

        public object DBValue(DBMSTypes DBMSType)
        {
            if ((Value == null || Value.Equals(DBNull.Value)) && this.AllowNull)
                return DBNull.Value;

            switch (SQLType)
            {
                case SQLBuilderTypes.sqlString:
                case SQLBuilderTypes.sqlChar:
                    string strValue = Value.SafeString();

                    if (strValue.Length == 0 && this.AllowNull)
                        return DBNull.Value;

                    if (MaxLen > 0)
                        strValue = Strings.StrLeft(strValue, MaxLen);

                    return strValue;

                case SQLBuilderTypes.sqlInt:
                    if (Value.IsNumeric())
                        return Value.SafeInt();
                    else
                    {
                        if (AllowNull)
                            return DBNull.Value;
                        else
                            return DefaultValue.SafeInt();
                    }

                case SQLBuilderTypes.sqlFloat:                
                    if (Value.IsNumeric())
                        return Value.SafeDouble();
                    else
                    {
                        if (AllowNull)
                            return DBNull.Value;
                        else
                            return DefaultValue.SafeDouble();
                    }

                case SQLBuilderTypes.sqlMoney:
                    if (Value.IsNumeric())
                        return Value.SafeDecimal();
                    else
                    {
                        if (AllowNull)
                            return DBNull.Value;
                        else
                            return DefaultValue.SafeDecimal();
                    }

                case SQLBuilderTypes.sqlBool:
                    if (DBMSType == DBMSTypes.MSSQLServer)
                        return Value.SafeBool().ToInt();
                    else
                        return Value.SafeBool();


                case SQLBuilderTypes.sqlDate:
                    if (Value.IsDate())
                    {
                        if (AllowNull && Value.SafeDateTime() == DateTime.MinValue)
                            return DBNull.Value;
                        else
                            return Value;
                    }
                    else
                    {
                        if (AllowNull || !DefaultValue.IsDate())
                            return DBNull.Value;
                        else
                            return Value.SafeDateTime();
                    }

                case SQLBuilderTypes.sqlMemo:
                    string strValueMemo = Value.SafeString();

                    if (strValueMemo.Length == 0 && this.AllowNull)
                        return DBNull.Value;

                    return strValueMemo;

                default:
                    return DBNull.Value;
            }

        }
    }


    public class SQLBuilder : DisposableObject
    {
        private string pTablename;

        private Dictionary<string, SQLColumnDefinition> Columns;
        private List<string> ColumnNames;        
        private DBMSTypes pDBMSType = DBMSTypes.MSSQLServer;
        public SQLServerBoolTypes SQLServerBoolType = SQLServerBoolTypes.boolTinyInt;

        private System.Globalization.NumberFormatInfo nfi;

        public System.Data.SqlClient.SqlConnection SQLConn;        
        private System.Data.SqlClient.SqlCommand cmdSQLInsert = null;
        private System.Data.SqlClient.SqlCommand cmdSQLUpdate = null;
        private System.Data.SqlClient.SqlCommand cmdSQLDelete = null;

        public System.Data.OleDb.OleDbConnection OLEDBConn = null;
        private System.Data.OleDb.OleDbCommand cmdOLEDBInsert = null;
        private System.Data.OleDb.OleDbCommand cmdOLEDBUpdate = null;
        private System.Data.OleDb.OleDbCommand cmdOLEDBDelete = null;
        
        private System.Data.SqlClient.SqlTransaction m_SQLTrans = null;
        public System.Data.SqlClient.SqlTransaction SQLTrans
        {
            get
            {
                return m_SQLTrans;
            }
            set
            {
                if (m_SQLTrans != value)
                {
                    m_SQLTrans = value;
                    
                    if (cmdSQLInsert != null)                    
                        cmdSQLInsert.Transaction = value;
                    
                    if (cmdSQLUpdate != null)                    
                        cmdSQLUpdate.Transaction = value;
                    
                    if (cmdSQLDelete != null)
                        cmdSQLDelete.Transaction = value;                    
                }
            }
        }


        private System.Data.OleDb.OleDbTransaction m_OLEDBTrans = null;
        public System.Data.OleDb.OleDbTransaction OLEDBTrans
        {
            get
            {
                return m_OLEDBTrans;
            }
            set
            {
                if (m_OLEDBTrans != value)
                {
                    m_OLEDBTrans = value;

                    if (cmdOLEDBInsert != null)
                        cmdOLEDBInsert.Transaction = value;

                    if (cmdOLEDBUpdate != null)
                        cmdOLEDBUpdate.Transaction = value;

                    if (cmdOLEDBDelete != null)
                        cmdOLEDBDelete.Transaction = value;                    
                }
            }
        }
        

        public SQLBuilder(string Tablename, IDbConnection conn, IDbTransaction trans)
        {
            pTablename = Tablename;
            Columns = new Dictionary<string, SQLColumnDefinition>();
            ColumnNames = new List<string>();

            if (conn != null)
            {
                if (conn is SqlConnection)
                {
                    pDBMSType = DBMSTypes.MSSQLServer;
                    SQLConn = (SqlConnection)conn;
                }
                else if (conn is OleDbConnection)
                {
                    pDBMSType = DBMSTypes.MSAccess;
                    OLEDBConn = (OleDbConnection)conn;
                }
            }

            if (trans != null)
            {
                if (trans is SqlTransaction)
                    SQLTrans = (SqlTransaction)trans;
                else if (trans is OleDbTransaction)
                    OLEDBTrans = (OleDbTransaction)trans;
            }

            nfi = new System.Globalization.CultureInfo("en-US", false).NumberFormat;
            nfi.NumberGroupSeparator = "";
        }

        public void ResetColumns(string Tablename)
        {
            Columns.Clear();
            ColumnNames.Clear();

            if (Tablename != "")
            {
                pTablename = Tablename;
            }

            ResetParameters();
        }

        public void ResetData()
        {
            ResetData(true);
        }

        public void ResetData(bool resetParameters)
        {
            foreach (SQLColumnDefinition d in Columns.Values)
            {
                d.Value = null;                
            }

            if (resetParameters)
                ResetParameters();
        }

        private void ResetParameters()
        {
            if (cmdSQLInsert != null)
            {                
                cmdSQLInsert.Dispose();
                cmdSQLInsert = null;
            }

            if (cmdOLEDBInsert != null)
            {
                cmdOLEDBInsert.Dispose();
                cmdOLEDBInsert = null;
            }

            if (cmdSQLUpdate != null)
            {
                cmdSQLUpdate.Dispose();
                cmdSQLUpdate = null;
            }

            if (cmdOLEDBUpdate != null)
            {
                cmdOLEDBUpdate.Dispose();
                cmdOLEDBUpdate = null;
            }

            if (cmdSQLDelete != null)
            {
                cmdSQLDelete.Dispose();
                cmdSQLDelete = null;
            }

            if (cmdOLEDBDelete != null)
            {
                cmdOLEDBDelete.Dispose();
                cmdOLEDBDelete = null;
            }
        }

        public int GetActiveCount()
        {
            int i = 0;
            foreach (SQLColumnDefinition d in Columns.Values)
            {
                if (d.bActive)
                    i++;
            }

            return i;
        }        

        public void AddColumn(string ColumnName)
        {
            AddColumn(ColumnName, SQLBuilderTypes.sqlString, true, 255, "");
        }

        public void AddColumn(string ColumnName, SQLBuilderTypes SQLType)
        {
            AddColumn(ColumnName, SQLType, true, 255, "");
        }

        public void AddColumn(string ColumnName, SQLBuilderTypes SQLType, bool AllowNull)
        {
            AddColumn(ColumnName, SQLType, AllowNull, 255, "");
        }

        public void AddColumn(string ColumnName, SQLBuilderTypes SQLType, bool AllowNull, int MaxLen)
        {
            AddColumn(ColumnName, SQLType, AllowNull, MaxLen, "");
        }

        public void AddColumn(string ColumnName, SQLBuilderTypes SQLType, bool AllowNull, int MaxLen, object DefaultValue)
        {
            SQLColumnDefinition d = new SQLColumnDefinition();
            d.SQLType = SQLType;
            d.AllowNull = AllowNull;
            d.MaxLen = MaxLen;
            d.Value = null;

            if (DefaultValue == null)
            {
                switch (SQLType)
                {
                    case SQLBuilderTypes.sqlString:
                    case SQLBuilderTypes.sqlMemo:
                    case SQLBuilderTypes.sqlChar:
                        d.DefaultValue = "";
                        break;

                    case SQLBuilderTypes.sqlBool:
                        d.DefaultValue = false;
                        break;

                    case SQLBuilderTypes.sqlInt:
                        d.DefaultValue = 0;
                        break;

                    case SQLBuilderTypes.sqlFloat:
                        d.DefaultValue = 0;
                        break;

                    case SQLBuilderTypes.sqlDate:
                        d.DefaultValue = new DateTime(1990, 1, 1);
                        break;
                }
            }
            else
            {
                d.DefaultValue = DefaultValue;
            }


            if (!Columns.ContainsKey(ColumnName))
            {
                Columns.Add(ColumnName, d);
                ColumnNames.Add(ColumnName);
            }
        }

        public bool AddValue(string ColumnName, object Data)
        {
            if (!Columns.ContainsKey(ColumnName))
                return false;

            SQLColumnDefinition d = Columns[ColumnName];
            if (d == null)
                return false;            

            d.Value = Data;
            return true;
        }

        public void ResetColumnsActive()
        {
            foreach (SQLColumnDefinition d in Columns.Values)
            {
                d.bActive = true;
                d.bWhere = false;
            }
        }

        public void SetColumnActive(string ColumnName, bool bActive)
        {
            if (Columns.ContainsKey(ColumnName))
                Columns[ColumnName].bActive = bActive;
        }

        public void SetColumnWhere(string ColumnName, bool bWhere)
        {
            if (Columns.ContainsKey(ColumnName))
                Columns[ColumnName].bWhere = bWhere;
        }

        public bool GetColumnActive(string ColumnName)
        {
            if (Columns.ContainsKey(ColumnName))
                return Columns[ColumnName].bActive;
            else
                return false;
        }

        public bool GetColumnWhere(string ColumnName)
        {
            if (Columns.ContainsKey(ColumnName))
                return Columns[ColumnName].bWhere;
            else
                return false;
        }

        public string InsertCommand()
        {
            System.Text.StringBuilder SB = new System.Text.StringBuilder();

            SB.Append("insert into ");
            SB.Append(pTablename);
            SB.Append(" (");

            SQLColumnDefinition d;

            bool bFlag = false;
            foreach (string f in ColumnNames)
            {
                d = Columns[f];
                if (d.bActive)
                {
                    if (bFlag)
                        SB.Append(", ");

                    SB.Append(f);
                    bFlag = true;
                }
            }

            SB.Append(") VALUES (");

            bFlag = false;
            foreach (string f in ColumnNames)
            {
                d = Columns[f];
                if (d.bActive)
                {
                    if (bFlag)
                        SB.Append(", ");

                    SB.Append(d.DBValueString(pDBMSType, nfi));
                    bFlag = true;
                }
            }

            SB.Append(")");

            return SB.ToString();
        }

        public string UpdateCommand(string WhereClause)
        {
            System.Text.StringBuilder SB = new System.Text.StringBuilder();

            SB.Append("UPDATE ");
            SB.Append(pTablename);
            SB.Append(" SET ");

            SQLColumnDefinition d;

            bool bFlag = false;
            foreach (string f in ColumnNames)
            {
                d = Columns[f];
                if (d.bActive)
                {

                    if (bFlag)
                        SB.Append(", ");

                    SB.Append(f);
                    SB.Append("=");

                    SB.Append(d.DBValueString(pDBMSType, nfi));
                    bFlag = true;
                }
            }

            SB.Append(" WHERE (");
            SB.Append(WhereClause);
            SB.Append(")");

            return SB.ToString();
        }

        public string DeleteCommand(string WhereClause)
        {
            System.Text.StringBuilder SB = new System.Text.StringBuilder();

            SB.Append("DELETE FROM ");
            SB.Append(pTablename);
            SB.Append(" WHERE (");
            SB.Append(WhereClause);
            SB.Append(")");

            return SB.ToString();
        }

        private void SetCommandParameters(ref System.Data.SqlClient.SqlCommand cmd)
        {
            SQLColumnDefinition def;

            foreach (string ColumnName in ColumnNames)
            {
                def = Columns[ColumnName];

                if (def.bActive || def.bWhere)
                {
                    switch (def.SQLType)
                    {
                        case SQLBuilderTypes.sqlString:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.SqlDbType.NVarChar, def.MaxLen);
                            break;

                        case SQLBuilderTypes.sqlInt:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.SqlDbType.Int);
                            break;

                        case SQLBuilderTypes.sqlDate:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.SqlDbType.DateTime);
                            break;

                        case SQLBuilderTypes.sqlFloat:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.SqlDbType.Float);
                            break;

                        case SQLBuilderTypes.sqlMemo:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.SqlDbType.NText);
                            break;

                        case SQLBuilderTypes.sqlBool:
                            if (SQLServerBoolType == SQLServerBoolTypes.boolTinyInt)
                                cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.SqlDbType.TinyInt);
                            else
                                cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.SqlDbType.Bit);
                            break;

                        case SQLBuilderTypes.sqlChar:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.SqlDbType.Char, def.MaxLen);
                            break;

                        case SQLBuilderTypes.sqlMoney:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.SqlDbType.Money);
                            break;
                    }
                }
            }
        }


        private void SetCommandParameters(ref System.Data.OleDb.OleDbCommand cmd)
        {
            SQLColumnDefinition def;

            foreach (string ColumnName in ColumnNames)
            {
                def = Columns[ColumnName];

                if (def.bActive || def.bWhere)
                {
                    switch (def.SQLType)
                    {
                        case SQLBuilderTypes.sqlString:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.OleDb.OleDbType.VarWChar, def.MaxLen);
                            break;

                        case SQLBuilderTypes.sqlInt:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.OleDb.OleDbType.Integer);
                            break;

                        case SQLBuilderTypes.sqlDate:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.OleDb.OleDbType.Date);
                            break;

                        case SQLBuilderTypes.sqlFloat:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.OleDb.OleDbType.Double);
                            break;

                        case SQLBuilderTypes.sqlMemo:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.OleDb.OleDbType.LongVarWChar);
                            break;

                        case SQLBuilderTypes.sqlBool:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.OleDb.OleDbType.Boolean);
                            break;

                        case SQLBuilderTypes.sqlChar:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.OleDb.OleDbType.WChar, def.MaxLen);
                            break;

                        case SQLBuilderTypes.sqlMoney:
                            cmd.Parameters.Add("@" + Column2ParameterName(ColumnName), System.Data.OleDb.OleDbType.Double);
                            break;
                    }
                }
            }
        }


        private string Column2ParameterName(string S)
        {
            if (S.Length > 1 && S[0] == '[' && S[S.Length - 1] == ']')
                return S.Substring(1, S.Length - 2);
            else
                return S;
        }

        // <<<<<<<<<<<<<<<<<<<<<<< DataSet >>>>>>>>>>>>>>>>>>>>>>>>>

        public bool InsertIntoDataSet(DataSet DS)
        {
            //int index = DS.Tables.IndexOf(null, pTablename);
            //DataTable dt = DS.Tables[index];
            DataRow row = DS.Tables[pTablename].NewRow();

            foreach (string ColumnName in ColumnNames)
            {
                SQLColumnDefinition d = Columns[ColumnName];
                if (d.bActive)
                {                    
                    row[ColumnName] = d.DBValue(pDBMSType);
                }
            }

            DS.Tables[pTablename].Rows.Add(row);
            return true;
        }

        private string ActiveColumnsList()
        {
            return String.Join(", ", ColumnNames.Where(p => Columns[p].bActive));
        }

        private string ActiveParameterNameList()
        {
            return String.Join(", ", ColumnNames.Where(p => Columns[p].bActive).Select(p => "@" + Column2ParameterName(p)));
        }

        private string ActiveColumnEqualParameterList()
        {
            return String.Join(" AND ", ColumnNames.Where(p => Columns[p].bActive).Select(p => p + "=@" + Column2ParameterName(p)));
        }

        private string ActiveColumnSetParameterList()
        {
            return String.Join(", ", ColumnNames.Where(p => Columns[p].bActive).Select(p => p + "=@" + Column2ParameterName(p)));
        }

        private string WhereColumnEqualParameterList()
        {
            return String.Join(" AND ", ColumnNames.Where(p => Columns[p].bWhere).Select(p => p + "=@" + Column2ParameterName(p)));
        }

        // <<<<<<<<<<<<<<<<<<<<<<< SQL Server >>>>>>>>>>>>>>>>>>>>>>>>>        
        public System.Data.SqlClient.SqlCommand SqlInsertCommandObject(bool Prepared, bool WithIfNotExist)
        {            
            if (cmdSQLInsert == null)
            {
                cmdSQLInsert = new System.Data.SqlClient.SqlCommand();
                cmdSQLInsert.Connection = SQLConn;
                if (m_SQLTrans != null)
                    cmdSQLInsert.Transaction = m_SQLTrans;

                StringBuilder SB = new StringBuilder();

                // --- if not exist --
                SB.Append("IF NOT EXISTS (SELECT ");
                SB.Append(ActiveColumnsList());
                SB.Append(" FROM ");
                SB.Append(pTablename);
                SB.Append(" WHERE ");
                SB.Append(ActiveColumnEqualParameterList());
                SB.Append(") ");                
                // --- if not exist --

                SB.Append("Insert into " + pTablename + " (");
                SB.Append(ActiveColumnsList());
                SB.Append(") Values (");
                SB.Append(ActiveParameterNameList());
                SB.Append(")");

                cmdSQLInsert.CommandText = SB.ToString();

                SetCommandParameters(ref cmdSQLInsert);

                if (Prepared)
                    cmdSQLInsert.Prepare();
            }

            return SqlInsertCommandObject(Prepared);
        }

        public System.Data.SqlClient.SqlCommand SqlInsertCommandObject(bool Prepared)
        {
            SQLColumnDefinition def;

            if (cmdSQLInsert == null)
            {
                cmdSQLInsert = new System.Data.SqlClient.SqlCommand();
                cmdSQLInsert.Connection = SQLConn;
                if (m_SQLTrans != null)
                    cmdSQLInsert.Transaction = m_SQLTrans;

                StringBuilder SB = new StringBuilder();

                SB.Append("Insert into " + pTablename + " (");
                SB.Append(ActiveColumnsList());
                SB.Append(") Values (");
                SB.Append(ActiveParameterNameList());
                SB.Append(")");

                cmdSQLInsert.CommandText = SB.ToString();

                SetCommandParameters(ref cmdSQLInsert);

                if (Prepared)
                    cmdSQLInsert.Prepare();
            }

            // Data
            foreach (string ColumnName in ColumnNames)
            {
                def = Columns[ColumnName];
                if (def.bActive)
                {
                    cmdSQLInsert.Parameters["@" + Column2ParameterName(ColumnName)].Value = def.DBValue(pDBMSType);
                }
            }

            return cmdSQLInsert;
        }


        public System.Data.SqlClient.SqlCommand SqlUpdateCommandObject()
        {
            SQLColumnDefinition def;

            if (cmdSQLUpdate == null)
            {
                cmdSQLUpdate = new System.Data.SqlClient.SqlCommand();
                cmdSQLUpdate.Connection = SQLConn;
                if (m_SQLTrans != null)
                    cmdSQLUpdate.Transaction = m_SQLTrans;

                StringBuilder SB = new StringBuilder();
                SB.Append("Update " + pTablename + " set ");
                SB.Append(ActiveColumnSetParameterList());

                SB.Append(" WHERE ");
                SB.Append(WhereColumnEqualParameterList());

                cmdSQLUpdate.CommandText = SB.ToString();

                SetCommandParameters(ref cmdSQLUpdate);
            }

            // Data
            foreach (string ColumnName in ColumnNames)
            {
                def = Columns[ColumnName];
                if (def.bActive)
                {
                    cmdSQLUpdate.Parameters["@" + Column2ParameterName(ColumnName)].Value = def.DBValue(pDBMSType);
                }
            }

            return cmdSQLUpdate;
        }

        public System.Data.SqlClient.SqlCommand SqlUpdateCommandObject(string strWhere)
        {
            SQLColumnDefinition def;

            if (cmdSQLUpdate == null)
            {
                cmdSQLUpdate = new System.Data.SqlClient.SqlCommand();
                cmdSQLUpdate.Connection = SQLConn;
                if (m_SQLTrans != null)
                    cmdSQLUpdate.Transaction = m_SQLTrans;

                StringBuilder SB = new StringBuilder();
                SB.Append("Update " + pTablename + " set ");
                SB.Append(ActiveColumnSetParameterList());

                SB.Append(" WHERE ");
                SB.Append(strWhere);

                cmdSQLUpdate.CommandText = SB.ToString();

                SetCommandParameters(ref cmdSQLUpdate);
            }
            else
            {
                // Nur WHERE ndern !
                int i = Strings.InStr(cmdSQLUpdate.CommandText, " WHERE ");
                cmdSQLUpdate.CommandText = Strings.StrLeft(cmdSQLUpdate.CommandText, i - 1) + " WHERE " + strWhere;
                cmdSQLUpdate.Transaction = m_SQLTrans;
            }

            // Data
            foreach (string ColumnName in ColumnNames)
            {
                def = Columns[ColumnName];
                if (def.bActive)
                {
                    cmdSQLUpdate.Parameters["@" + Column2ParameterName(ColumnName)].Value = def.DBValue(pDBMSType);
                }
            }

            return cmdSQLUpdate;
        }

        public System.Data.SqlClient.SqlCommand SqlDeleteCommandObject(bool Prepared)
        {
            SQLColumnDefinition def;

            if (cmdSQLDelete == null)
            {
                cmdSQLDelete = new System.Data.SqlClient.SqlCommand();
                cmdSQLDelete.Connection = SQLConn;
                if (m_SQLTrans != null)
                    cmdSQLDelete.Transaction = m_SQLTrans;

                string strSQL = "Delete from " + pTablename + " WHERE " + WhereColumnEqualParameterList();

                cmdSQLDelete.CommandText = strSQL;

                SetCommandParameters(ref cmdSQLDelete);

                if (Prepared)
                    cmdSQLDelete.Prepare();
            }

            // Data
            foreach (string ColumnName in ColumnNames)
            {
                def = Columns[ColumnName];
                if (def.bActive)
                {
                    cmdSQLDelete.Parameters["@" + Column2ParameterName(ColumnName)].Value = def.DBValue(pDBMSType);
                }
            }

            return cmdSQLDelete;
        }

        public System.Data.SqlClient.SqlCommand SqlDeleteCommandObject(string strWhere, bool Prepared)
        {
            if (cmdSQLDelete == null)
            {
                cmdSQLDelete = new System.Data.SqlClient.SqlCommand();
                cmdSQLDelete.Connection = SQLConn;
                if (m_SQLTrans != null)
                    cmdSQLDelete.Transaction = m_SQLTrans;

                string strSQL = "Delete from " + pTablename + " WHERE " + strWhere;

                cmdSQLDelete.CommandText = strSQL;

                if (Prepared)
                    cmdSQLDelete.Prepare();
            }
            else
            {
                // Nur WHERE ndern !
                int i = Strings.InStr(cmdSQLDelete.CommandText, " WHERE ");
                cmdSQLDelete.CommandText = Strings.StrLeft(cmdSQLDelete.CommandText, i - 1) + " WHERE " + strWhere;
                cmdSQLDelete.Transaction = m_SQLTrans;
            }

            return cmdSQLDelete;
        }


        // <<<<<<<<<<<<<<<<<<<<<<< OLE-DB >>>>>>>>>>>>>>>>>>>>>>>>

        public System.Data.OleDb.OleDbCommand OLEDBInsertCommandObject(bool Prepared)
        {
            SQLColumnDefinition def;

            if (cmdOLEDBInsert == null)
            {
                cmdOLEDBInsert = new System.Data.OleDb.OleDbCommand();
                cmdOLEDBInsert.Connection = OLEDBConn;
                if (m_OLEDBTrans != null)
                    cmdOLEDBInsert.Transaction = m_OLEDBTrans;

                StringBuilder SB = new StringBuilder();
                SB.Append("Insert into " + pTablename + " (");

                SQLColumnDefinition d;
                bool bFlag = false;

                foreach (string ColumnName in ColumnNames)
                {
                    d = Columns[ColumnName];
                    if (d.bActive)
                    {
                        if (bFlag)
                            SB.Append(", ");

                        SB.Append(ColumnName);
                        bFlag = true;
                    }
                }

                SB.Append(") Values (");

                bFlag = false;
                foreach (string ColumnName in ColumnNames)
                {
                    d = Columns[ColumnName];
                    if (d.bActive)
                    {
                        if (bFlag)
                            SB.Append(", ");

                        SB.Append("@" + Column2ParameterName(ColumnName));
                        bFlag = true;
                    }
                }

                SB.Append(")");

                cmdOLEDBInsert.CommandText = SB.ToString();

                SetCommandParameters(ref cmdOLEDBInsert);

                if (Prepared)
                    cmdOLEDBInsert.Prepare();
            }

            // Data
            foreach (string ColumnName in ColumnNames)
            {
                def = Columns[ColumnName];
                if (def.bActive)
                {
                    cmdOLEDBInsert.Parameters["@" + Column2ParameterName(ColumnName)].Value = def.DBValue(pDBMSType);
                }
            }

            return cmdOLEDBInsert;
        }

        public System.Data.OleDb.OleDbCommand OLEDBUpdateCommandObject(string strWhere)
        {
            SQLColumnDefinition def;

            if (cmdOLEDBUpdate == null)
            {
                cmdOLEDBUpdate = new System.Data.OleDb.OleDbCommand();
                cmdOLEDBUpdate.Connection = OLEDBConn;
                if (m_OLEDBTrans != null)
                    cmdOLEDBUpdate.Transaction = m_OLEDBTrans;

                StringBuilder SB = new StringBuilder();

                SB.Append("Update " + pTablename + " set ");

                SQLColumnDefinition d;
                bool bFlag = false;

                foreach (string ColumnName in ColumnNames)
                {
                    d = Columns[ColumnName];
                    if (d.bActive)
                    {
                        if (bFlag)
                            SB.Append(", ");

                        SB.Append(ColumnName);
                        SB.Append("=@" + Column2ParameterName(ColumnName));

                        bFlag = true;
                    }
                }

                SB.Append(" WHERE ");
                SB.Append(strWhere);

                cmdOLEDBUpdate.CommandText = SB.ToString();

                SetCommandParameters(ref cmdOLEDBUpdate);
            }
            else
            {
                // Nur WHERE ndern !
                int i = Strings.InStr(cmdOLEDBUpdate.CommandText, " WHERE ");
                cmdOLEDBUpdate.CommandText = Strings.StrLeft(cmdOLEDBUpdate.CommandText, i - 1) + " WHERE " + strWhere;
                cmdOLEDBUpdate.Transaction = m_OLEDBTrans;
            }

            // Data
            foreach (string ColumnName in ColumnNames)
            {
                def = Columns[ColumnName];
                if (def.bActive)
                {
                    cmdOLEDBUpdate.Parameters["@" + Column2ParameterName(ColumnName)].Value = def.DBValue(pDBMSType);
                }
            }

            return cmdOLEDBUpdate;
        }

        public System.Data.OleDb.OleDbCommand OLEDBDeleteCommandObject(string strWhere, bool Prepared)
        {
            if (cmdOLEDBDelete == null)
            {
                cmdOLEDBDelete = new System.Data.OleDb.OleDbCommand();
                cmdOLEDBDelete.Connection = OLEDBConn;
                if (m_OLEDBTrans != null)
                    cmdOLEDBDelete.Transaction = m_OLEDBTrans;

                string strSQL = "Delete from " + pTablename + " WHERE " + strWhere;

                cmdOLEDBDelete.CommandText = strSQL;

                if (Prepared)
                    cmdOLEDBDelete.Prepare();
            }
            else
            {
                // Nur WHERE ndern !
                int i = Strings.InStr(cmdOLEDBDelete.CommandText, " WHERE ");
                cmdOLEDBDelete.CommandText = Strings.StrLeft(cmdOLEDBDelete.CommandText, i - 1) + " WHERE " + strWhere;
                cmdOLEDBDelete.Transaction = m_OLEDBTrans;
            }

            return cmdOLEDBDelete;
        }                

        // ********* IDisposable **********

        protected override void CleanupManagedResources()
        {
            // CleanUp here and set to null
            if (cmdSQLInsert != null)
            {
                cmdSQLInsert.Dispose();
                cmdSQLInsert = null;
            }

            if (cmdSQLUpdate != null)
            {
                cmdSQLUpdate.Dispose();
                cmdSQLUpdate = null;
            }

            if (cmdSQLDelete != null)
            {
                cmdSQLDelete.Dispose();
                cmdSQLDelete = null;
            }

            if (cmdOLEDBInsert != null)
            {
                cmdOLEDBInsert.Dispose();
                cmdOLEDBInsert = null;
            }

            if (cmdOLEDBUpdate != null)
            {
                cmdOLEDBUpdate.Dispose();
                cmdOLEDBUpdate = null;
            }

            if (cmdOLEDBDelete != null)
            {
                cmdOLEDBDelete.Dispose();
                cmdOLEDBDelete = null;
            }
        }

        //private bool disposed = false;
        //public void Dispose()
        //{
        //    Dispose(true);
        //    GC.SuppressFinalize(this);
        //}
        //// NOTE: Leave out the finalizer altogether if this class doesn't 
        //// own unmanaged resources itself, but leave the other methods
        //// exactly as they are. 
        ////~SQLBuilder()
        ////{
        ////    // Finalizer calls Dispose(false)
        ////    Dispose(false);            
        ////}
        //// The bulk of the clean-up code is implemented in Dispose(bool)
        //protected virtual void Dispose(bool disposing)
        //{
        //    if (!this.disposed)
        //    {
        //        if (disposing)
        //        {
        //            // CleanUp here and set to null
        //            if (cmdSQLInsert != null)
        //            {
        //                cmdSQLInsert.Dispose();
        //                cmdSQLInsert = null;
        //            }

        //            if (cmdSQLUpdate != null)
        //            {
        //                cmdSQLUpdate.Dispose();
        //                cmdSQLUpdate = null;
        //            }

        //            if (cmdSQLDelete != null)
        //            {
        //                cmdSQLDelete.Dispose();
        //                cmdSQLDelete = null;
        //            }

        //            if (cmdOLEDBInsert != null)
        //            {
        //                cmdOLEDBInsert.Dispose();
        //                cmdOLEDBInsert = null;
        //            }

        //            if (cmdOLEDBUpdate != null)
        //            {
        //                cmdOLEDBUpdate.Dispose();
        //                cmdOLEDBUpdate = null;
        //            }

        //            if (cmdOLEDBDelete != null)
        //            {
        //                cmdOLEDBDelete.Dispose();
        //                cmdOLEDBDelete = null;
        //            }
        //        }
        //    }
        //    disposed = true;
        //}
    }

}
